export const normalizeTemplate = (nodes) => {
    const n = [];
    let text = "";
    for (let i = 0; i < nodes.length; i++) {
        const c = nodes[i];
        switch (c.type) {
            case 0 /* INodeType.Element */:
                if (text !== "") {
                    n.push({
                        type: 1 /* INodeType.Text */,
                        value: text,
                    });
                    text = "";
                }
                n.push({
                    ...c,
                    children: normalizeTemplate(c.children),
                });
                break;
            case 2 /* INodeType.Expr */:
                if (text !== "") {
                    n.push({
                        type: 1 /* INodeType.Text */,
                        value: text,
                    });
                    text = "";
                }
                n.push(c);
                break;
            case 1 /* INodeType.Text */:
                text += c.value;
        }
    }
    if (text !== "") {
        n.push({
            type: 1 /* INodeType.Text */,
            value: text,
        });
    }
    return n;
};
export const createSNode = (node, flags) => {
    const properties = node.properties;
    const iChildren = node.children;
    let propsExprs = 0;
    for (let i = 0; i < properties.length; i++) {
        if (!isStaticProperty(properties[i])) {
            propsExprs++;
        }
    }
    let children = null;
    let siblingsFlags = 0;
    let childrenExprs = 0;
    let i = iChildren.length;
    if (i > 0) {
        children = Array(i);
        while (i > 0) {
            const child = iChildren[--i];
            switch (child.type) {
                case 0 /* INodeType.Element */:
                    const sNode = createSNode(child, siblingsFlags);
                    if (sNode.flags & 1 /* SNodeFlags.HasExpressions */) {
                        flags |= 1 /* SNodeFlags.HasExpressions */;
                        siblingsFlags |= 2 /* SNodeFlags.HasNextExpressions */;
                    }
                    siblingsFlags |= 4 /* SNodeFlags.HasNextDOMNode */;
                    children[i] = sNode;
                    break;
                case 2 /* INodeType.Expr */:
                    siblingsFlags |= 2 /* SNodeFlags.HasNextExpressions */;
                    childrenExprs++;
                    children[i] = {
                        node: child,
                        stateIndex: 0,
                        children: null,
                        childrenExprs: 0,
                        propsExprs: 0,
                        flags: siblingsFlags,
                    };
                    break;
                case 1 /* INodeType.Text */:
                    children[i] = {
                        node: child,
                        stateIndex: 0,
                        children: null,
                        childrenExprs: 0,
                        propsExprs: 0,
                        flags: siblingsFlags,
                    };
                    siblingsFlags |= 4 /* SNodeFlags.HasNextDOMNode */;
            }
        }
    }
    if (propsExprs > 0 || childrenExprs > 0) {
        flags |= 1 /* SNodeFlags.HasExpressions */;
    }
    return {
        node,
        stateIndex: 0,
        children,
        childrenExprs,
        propsExprs,
        flags,
    };
};
export const isStaticProperty = (prop) => ((typeof prop.value !== "number") ||
    (prop.type === 0 /* IPropertyType.Attribute */ &&
        prop.key === "class" &&
        prop.hoist === true));
const HTML_VOID_ELEMENTS = (/^(embed|input|param|source|track|area|base|link|meta|br|col|hr|img|wbr)$/);
export const isVoidElement = (tag) => HTML_VOID_ELEMENTS.test(tag);
//# sourceMappingURL=shared.js.map